from django.db import models
from django.contrib.auth.models import User


class Project(models.Model):
    title = models.CharField(max_length=200)
    professor = models.ForeignKey(User, on_delete=models.CASCADE)
    description = models.CharField(max_length=2048)
    group_size = models.CharField(max_length=128)  # TODO: Not implemented
    status = models.CharField(max_length=128)
    date_created = models.DateTimeField(
        auto_now_add=True, name='date_created')  # TODO: Not implemented
    last_updated = models.DateTimeField(
        auto_now_add=True, name='last_updated')  # TODO: Not implemented
    # Tags - see ProjectTag below

    def __str__(self):
        return f'{self.title}, by {self.professor}. Status: {self.status}'


class Application(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE)
    student = models.ForeignKey(User, on_delete=models.CASCADE)
    message = models.CharField(max_length=2000)
    date_created = models.DateTimeField(auto_now_add=True, name='date_created')
    last_updated = models.DateTimeField(
        auto_now_add=True, name='last_updated')  # TODO: Not implemented
    priority = models.IntegerField(default=None, blank=True, null=True)
    professor_status = models.CharField(max_length=32)  # TODO: Not implemented
    student_status = models.CharField(max_length=32)  # TODO: Not implemented

    def __str__(self):
        return f'{self.student} on {self.project}'


class TagCategory(models.Model):
    category = models.CharField(max_length=128)

    def __str__(self):
        return self.category


class Tag(models.Model):
    name = models.CharField(max_length=256, unique=True)
    category = models.ForeignKey(
        TagCategory, on_delete=models.SET_NULL, blank=True, null=True, default=None)
    class_ids = models.CharField(
        max_length=256, blank=True, null=True, default=None)  # TODO: not implemented

    def __str__(self):
        return self.name


class ProjectTag(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE)
    tag = models.ForeignKey(Tag, on_delete=models.CASCADE)

    def __str__(self):
        return f'{self.project.title}: {self.tag.name}'


class StudentTag(models.Model):
    # TODO: not implemented
    student = models.ForeignKey(User, on_delete=models.CASCADE)
    tag = models.ForeignKey(Tag, on_delete=models.CASCADE)

    def __str__(self):
        return f'{self.student.first_name} {self.student.last_name}: {self.tag.name}'
